#include "pull.h"
#include "args.h"
#include "config.h"
#include "error.h"
#include "log.h"
#include "paths.h"
#include "url.h"
#include "util.h"

#include <git2.h>
#include <git2/global.h>
#include <libintl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>

#define _(x) gettext(x)

int pull_progress(const git_indexer_progress *stats, void *data){
  bar(stats->indexed_objects, stats->total_objects);
  return 0;
}

bool pull_cmd() {
  if (args.count != 2) {
    error(_("Please specify a config name or a URL"));
    return false;
  }

  char *repo_url = NULL, *repo_root = NULL;
  config_t repo_cfg = {.name = NULL};
  bool  ret = false;

  if (url_is_local(args.list[1])) {
    repo_root = args.list[1];
    goto COPY;
  }

  else if (url_is_name(args.list[1]))
    repo_url = url_complete(args.list[1]);

  else
    repo_url = url_ensure_protocol(args.list[1]);

  rmrf(mc_tmp_path);
  repo_root = mc_tmp_path;

  git_libgit2_init();

  git_repository   *repo                      = NULL;
  git_clone_options opts                      = GIT_CLONE_OPTIONS_INIT;
  opts.fetch_opts.callbacks.transfer_progress = pull_progress;
  opts.fetch_opts.callbacks.payload = NULL;

  info(_("Cloning %s"), repo_url);
  bar_init();

  if (git_clone(&repo, repo_url, repo_root, &opts) < 0) {
    const git_error *e = git_error_last();
    error(_("Failed to clone the %s:"), repo_url);
    details(e->message);
    bar_free();
    goto END;
  }

  bar_free();
  git_libgit2_shutdown();

COPY:
  if (chdir(repo_root) < 0) {
    error(_("Failed to chdir to %s"), repo_root);
    goto END;
  }

  if (!config_load(&repo_cfg, "mc.cfg")) {
    error(_("Failed to load the configuration file (mc.cfg):"));
    details(errch);
    goto END;
  }

  success(_("Loaded repository configuration"));
  info(_("Configuration details:\n"));
  printf(COLOR_BOLD "    %s     " COLOR_RESET "=> %s\n", _("Name"), repo_cfg.name);
  printf(COLOR_BOLD "    %s   " COLOR_RESET "=> %s\n", _("Author"), repo_cfg.author);

  printf(COLOR_BOLD "    %s " COLOR_RESET "=> ", _("Keywords"));
  for (int i = 0; i < repo_cfg.keywords.s; i++)
    printf("%s ", repo_cfg.keywords.c[i]);
  printf("\n\n");

  if(!yesno(_("Do you want to continue?")))
    goto END;

  info(_("Checking all the targets"));
  target_t *cur = repo_cfg.t_first;
  while(NULL != cur){
    if(!exists(cur->src)){
      error(_("Failed to access the source for the target \"%s\""), cur->name);
      goto END;
    }
    cur = cur->next;
  }

  success(_("All the targets are OK"));

END:
  if(repo_cfg.name != NULL)
    config_free(&repo_cfg);
  free(repo_url);
  return ret;
}
